%% sPADfit
%This program extracts numerical quantities from the simplified activity
%distribution data for each intermetallic particle being analysed.

%Main input
    %folder containing numerical data corresponding to the simplified pixel
    %activity distribution for each intermetallic particle RROI

%Main output
    %csv file containing extracted quantities
    %plots visualizing the fit

%% Import inputs

clc; clear;

%Import processed affected area numerical data (1 sec interval)
dir1=uigetdir('C:\','Select folder containing INPUT histogram data (dt=1 s)');
if dir1==0
    errordlg('Program Terminated. No input folder specified.');
    return
end

%Import processed affected area numerical data (1 sec interval)
dir10=uigetdir('C:\','Select folder containing INPUT histogram data (dt=10 s)');
if dir10==0
    errordlg('Program Terminated. No input folder specified.');
    return
end

%Input parameters used for kinetic parameter calculation
prompt={'Data File Type','ROI Margin (pix)','Min Activity Threshold Used','Duration (s)','t supply','max GL','stat range','conf level','tol','midGL at specific time','qs tol'};
dlgtitle='Input kinetic calculation parameters';
dims=[1 50];
definput={'csv','4','5','10800','0','255','5','0.005','1e-3','180','1e-3'};
answer=inputdlg(prompt,dlgtitle,dims,definput);

if isempty(answer)
    errordlg('Program terminated. No calculation parameters specified.');
    return
else
    FT=answer(1); lft=length(FT);
    w=str2double(answer(2));
    th=str2double(answer(3));
    dw=str2double(answer(4));
    tsupply=str2double(answer(5));
    uGL=str2double(answer(6));
    sw=str2double(answer(7));
    CL=str2double(answer(8));
    sltol=str2double(answer(9));
    sptm=str2double(answer(10));
    qstol=str2double(answer(11));
end

%Output folder
outpath=uigetdir('C:\','Select folder for OUTPUT file');
if outpath==0
    errordlg('Program Terminated. No output folder specified.');
    return
end

%Output Data
dirn=['InhbKntcs_GL-based_v8_tol_' num2str(qstol)];
fs0=fullfile(outpath,dirn);
mkdir(fs0)

%Process Numerical Data
%Histogram directory 1 sec interval
listH=dir(fullfile(dir1,['*' char(FT)]));
LH=length(listH);

%Histogram directory10 sec interval
listH10=dir(fullfile(dir10,['*' char(FT)]));
LH10=length(listH10);

%Setup array output
HistP=cell(LH+1,11);
HistP{1,1}='IM ROI ID';
HistP{1,2}='toi_maxGL';
HistP{1,3}='LTG';
HistP{1,4}='toi_midGL';
HistP{1,5}='midGL3';
HistP{1,6}='midGL f';
HistP{1,7}='EqS';
HistP{1,8}='maxGL3';
HistP{1,9}=['midGL_' num2str(sptm)];
HistP{1,10}='t_qs';
HistP{1,11}='qs_slope';

hct=1;
hct_10=1;

dirn_plotfig='InhbKntcPlot_GL';
fs1=fullfile(outpath,dirn,dirn_plotfig);
mkdir(fs1)

for i=1:LH
    %Update count
    hct=hct+1;
    
    %import 1s data
    hg=listH(i);
    hgn=[hg.folder '\' hg.name];
    hgd=csvread(hgn);
    [hr,hc]=size(hgd);
    ht=hgd(:,262);
    htp=ht(2:end); %remove first data point (t=0)
    gl=hgd(:,259);
    glp=gl(2:end);
    glp_sm=medfilt1(glp,sw,'truncate'); %median filter on an sw data point window
    lhs=length(glp_sm);
    
    %import 10s data
    hg10=listH10(i);
    hgn10=[hg10.folder '\' hg10.name];
    hgd10=csvread(hgn10);
    [hr10,hc10]=size(hgd10);
    ht10=hgd10(:,262);
    ht10p=ht10(2:end); %remove first data point (t=0)
    gl10=hgd10(:,259);
    gl10p=gl10(2:end);
    gl10p_sm=medfilt1(gl10p,sw,'truncate'); %median filter on an sw data point window
    lhs10=length(gl10p_sm);
    
    %maxGL at 3h
    maxGL3=mean(gl10p_sm(lhs10-5:lhs10));
    HistP{hct,8}=maxGL3;
    
    %search for estimation range
    cd=0;
    for j=1+sw:sw:lhs-sw+1
        tv=ttest(glp_sm(1:sw),glp_sm(j:j+sw-1),'Alpha',CL);
        pp=j+sw-1; Ap=(glp_sm(pp)-glp_sm(1))/(htp(pp)-htp(1));
        if tv==1 && Ap>=sltol
            cd=1; p2=j+sw-1; break
        end
    end
    
    if cd==1 %apply triangle method
        A=(glp_sm(p2)-glp_sm(1))/(htp(p2)-htp(1));
        B=-1;
        C=glp_sm(p2)-A*(htp(p2));
        x1=htp(1:p2); y1=glp_sm(1:p2);
        dist_1=abs(A.*x1+B.*y1+C)./sqrt(A*A+B*B);
        dist_1=smooth(dist_1,sw,'sgolay');
        toi_maxgl_pos=find(dist_1==max(dist_1),1,'first');
        toi_maxgl=htp(toi_maxgl_pos);
    else
        for jj=1+sw:sw:lhs10-sw+1
            tv10=ttest(gl10p_sm(1:sw),gl10p_sm(jj:jj+sw-1),'Alpha',CL);
            pp2=jj+sw-1; Ap2=(gl10p_sm(pp2)-gl10p_sm(1))/(ht10p(pp2)-ht10p(1));
            if tv10==1 && Ap2>=sltol
                cd=10;
                p2=jj+sw-1;
                
                %apply triangle method
                A=(gl10p_sm(p2)-gl10p_sm(1))/(ht10p(p2)-ht10p(1));
                B=-1;
                C=gl10p_sm(p2)-A*(ht10p(p2));
                x1=ht10p(1:p2); y1=gl10p_sm(1:p2);
                dist_1=abs(A.*x1+B.*y1+C)./sqrt(A*A+B*B);
                dist_1=smooth(dist_1,sw,'sgolay');
                toi_maxgl_pos=find(dist_1==max(dist_1),1,'first');
                toi_maxgl=ht10p(toi_maxgl_pos);
                break
            else
                cd=0;
            end
        end
    end
    
    %LTG
    if cd==1
        ltg=mean(glp_sm(1:toi_maxgl_pos));
    elseif cd==10
        ltg=mean(gl10p_sm(toi_maxgl_pos));
    elseif cd==0
        ltg=mean(gl10p_sm);
        toi_maxgl=NaN;
    end
    
    %update array
    HistP{hct,1}=hg.name;
    HistP{hct,2}=toi_maxgl;
    HistP{hct,3}=ltg;
    
    %midGL analysis
    %1 s data
    glmid=hgd(:,263);
    glmid_p=glmid(2:end);
    glmid_psm=medfilt1(glmid_p,sw,'truncate');
    
    %midGL at time t
    spt_pos=find(htp>=sptm,1,'first');
    midGL_t=glmid_psm(spt_pos);
    HistP{hct,9}=midGL_t;
    
    %10 s data
    glmid10=hgd10(:,263);
    glmid10_p=glmid10(2:end);
    glmid10_psm=medfilt1(glmid10_p,sw,'truncate');
    
    toi_midgl_pos=find(glmid_psm>ltg,1,'first');
    toi_midgl10_pos=find(glmid10_psm>ltg,1,'first');
    
    if isempty(toi_midgl_pos) && isempty(toi_midgl10_pos)
        toi_midgl=NaN;
    elseif ~isempty(toi_midgl_pos)
        toi_midgl=htp(toi_midgl_pos);
    elseif ~isempty(toi_midgl10_pos)
        toi_midgl=ht10p(toi_midgl10_pos);
    end
    
    HistP{hct,4}=toi_midgl;
    
    %midGL @ 3h
    midGL3=mean(glmid10_psm(lhs10-max(sw,30)+1:lhs10));
    HistP{hct,5}=midGL3;
    
    %midGLf and Eqs
    %area under the curve for entire duration divided by duration*midGL3
    %Integration
    hsnz=glmid10_psm;
    htnz=ht10p;
    ls=length(hsnz);
    
    tt=htnz(ls)-htnz(1);
    aa=0;
    for jjj=2:ls
        aa=aa+0.5.*(hsnz(jjj-1)+hsnz(jjj)).*(htnz(jjj)-htnz(jjj-1));
    end
    f=aa/(midGL3*tt);
    EqS=f*(midGL3/tt);
    
    HistP{hct,6}=f;
    HistP{hct,7}=EqS;
    
    
    %quasi-steady onset
    cf=3;
    qsonset_pre=zeros(3,1);
    stsw=max(sw,60); %last 10 minutes
    swqs=max(sw,6); %1 minute intervals
    qs_stat=zeros(3,1);
    
    %max
    maxqs_pre=gl10p_sm;
    maxqs=medfilt1(maxqs_pre,cf*swqs,'truncate');
    
    %find onset of slow period
    if mean(maxqs)<1
        qs_stat(1)=1;
        qsonset_pre(1)=ht10p(1);
    else
        
        %estimate inflection point
        maxyslp=(mean(maxqs(lhs10-4:lhs10))-maxqs(1))/(ht10p(lhs10)-ht10p(1));
        maxyb=maxqs(1)-maxyslp*ht10p(1);
        maxy=maxyslp*ht10p+maxyb;
        maxy_stat=(maxqs-maxy)./abs(maxqs-maxy);
        
        %apply triangle method
        A1=maxyslp; B1=-1; C1=maxyb;
        max_dist=(abs(A1.*ht10p+B1.*maxqs+C1)./sqrt(A1*A1+B1*B1)).*maxy_stat;
        max_dist=smooth(max_dist,sw,'sgolay');
        max_qs_pos=find(max_dist==max(max_dist),1,'first');
        
        if ~isempty(max_qs_pos)
            qs_stat(1)=1;
            qsonset_pre(1)=ht10p(max_qs_pos);
        end
    end

    %min
    mingl10=hgd10(:,258);
    mingl10p=mingl10(2:end);
    minqs_pre=medfilt1(mingl10p,sw,'truncate');
    minqs=medfilt1(minqs_pre,cf*swqs,'truncate');
    
    %find onset of slow period
    if mean(minqs)<1
        qs_stat(2)=1;
        qsonset_pre(2)=ht10p(1);
    else
        %estimate inflection point
        minyslp=(mean(minqs(lhs10-4:lhs10))-minqs(1))/(ht10p(lhs10)-ht10p(1));
        minyb=minqs(1)-minyslp*ht10p(1);
        miny=minyslp*ht10p+minyb;
        miny_stat=(minqs-miny)./abs(minqs-miny);
        
        %apply triangle method
        A2=minyslp; B2=-1; C2=minyb;
        min_dist=(abs(A2.*ht10p+B2.*minqs+C2)./sqrt(A2*A2+B2*B2)).*miny_stat;
        min_dist=smooth(min_dist,sw,'sgolay');
        min_qs_pos=find(min_dist==max(min_dist),1,'first');
        
        if ~isempty(min_qs_pos)
            qs_stat(2)=1;
            qsonset_pre(2)=ht10p(min_qs_pos);
        end
    end
    
    %mid
    midqs_pre=glmid10_psm;
    midqs=medfilt1(midqs_pre,cf*swqs,'truncate');
    
    %find onset of slow period
    if mean(midqs)<1
        qs_stat(3)=1;
        qsonset_pre(3)=ht10p(1);
    else
        %estimate inflection point
        midyslp=(mean(midqs(lhs10-4:lhs10))-midqs(1))/(ht10p(lhs10)-ht10p(1));
        midyb=midqs(1)-midyslp*ht10p(1);
        midy=midyslp*ht10p+midyb;
        midy_stat=(midqs-midy)./abs(midqs-midy);
        
        %apply triangle method
        A3=midyslp; B3=-1; C3=midyb;
        mid_dist=(abs(A3.*ht10p+B3.*midqs+C3)./sqrt(A3*A3+B3*B3)).*midy_stat;
        mid_dist=smooth(mid_dist,sw,'sgolay');
        mid_qs_pos=find(mid_dist==max(mid_dist),1,'first');
        
        if ~isempty(mid_qs_pos)
            qs_stat(3)=1;
            qsonset_pre(3)=ht10p(mid_qs_pos);
        end
    end
    
    if sum(qs_stat)==3
        %quasi steady evaluation
        qs_onset=max(qsonset_pre);
        qs_slope=qstol;
    else
        qs_onset=10800;
        qs_slope=NaN;
    end
    
    HistP{hct,10}=qs_onset+10;
    HistP{hct,11}=qs_slope;
    
    %plot midGL and qs
    close all;
    fg=figure('visible','off');
    
    if ~isnan(qs_onset)
        plot(ht10,glmid10,'k',htnz,hsnz,'r',ones(1,100)*tsupply,linspace(0,255,100),'k--',...
            ones(1,100)*qs_onset,linspace(0,255,100),'g--',ht10p,maxqs,ht10p,minqs)
        xlabel('time(s)');
        ylabel('midGL');
        ylim([0 uGL]);
        xlim([0 dw])
        xticks(0:dw/6:dw);
        hold on;
        xline(htnz(1),'r--','t_{onset}','LabelHorizontalAlignment','center',...
            'LabelOrientation','horizontal');
        yline(midGL3,'r--');
        
        ind=(htnz>=htnz(1));
        hs1i=hsnz;
        hs1i(~ind)=midGL3;
        area(htnz,hs1i,'EdgeColor','none','FaceColor','r')
        alpha(.3);
    else
        plot(ht10,glmid10,'k',htnz,hsnz,'r',ones(1,100)*tsupply,linspace(0,255,100),'k--',...
            ht10p,maxqs,ht10p,minqs)
        xlabel('time(s)');
        ylabel('midGL');
        ylim([0 uGL]);
        xlim([0 dw])
        xticks(0:dw/6:dw);
        hold on;
        xline(htnz(1),'r--','t_{onset}','LabelHorizontalAlignment','center',...
            'LabelOrientation','horizontal');
        yline(midGL3,'r--');
        
        ind=(htnz>=htnz(1));
        hs1i=hsnz;
        hs1i(~ind)=midGL3;
        area(htnz,hs1i,'EdgeColor','none','FaceColor','r')
        alpha(.3);
    end
    
    hold off;
    
    %save plot
    pn=[hg.name(1:(length(hg.name)-4)) '.jpg'];
    sn=fullfile(fs1,pn);
    exportgraphics(fg,sn,'Resolution',600);
    
    %plot maxGL
    close all;
    fg2=figure('visible','off');
    
    if~isnan(toi_maxgl)
        if cd==1
            plot(ht,gl,'k',htp,glp_sm,'r',ones(1,100)*tsupply,linspace(0,255,100),'k--')
            xlabel('time(s)');
            ylabel('maxGL');
            ylim([0 uGL]);
            xlim([0 600])
            xticks(0:100:600);
            hold on;
            xline(toi_maxgl,'r--','t_{o-max}','LabelHorizontalAlignment','center',...
                'LabelOrientation','horizontal');
            
            %save plot
            pn2=[hg.name(1:(length(hg.name)-4)) '_maxGL.jpg'];
            sn2=fullfile(fs1,pn2);
            exportgraphics(fg2,sn2,'Resolution',600);
        else
            plot(ht10,gl10,'k',ht10p,gl10p_sm,'r',ones(1,100)*tsupply,linspace(0,255,100),'k--')
            xlabel('time(s)');
            ylabel('maxGL');
            ylim([0 uGL]);
            xlim([0 dw])
            xticks(0:dw/6:dw);
            hold on;
            xline(toi_maxgl,'r--','t_{onset-max}','LabelHorizontalAlignment','center',...
                'LabelOrientation','horizontal');
            
            %save plot
            pn2=[hg.name(1:(length(hg.name)-4)) '_maxGL.jpg'];
            sn2=fullfile(fs1,pn2);
            exportgraphics(fg2,sn2,'Resolution',600);
        end
    end
    
    hold off
    
end

%output table
if hct<LH
    HistP(hct+1,:)=[];
end
nn=['Inhbkin_GLbased_' num2str(dw,'%0.0f') '_' num2str(sltol) '_seg.csv'];
sn1=fullfile(fs0,nn);
HistPt=cell2table(HistP(2:end,:),'VariableNames',HistP(1,:));
writetable(HistPt,sn1);